/*
 *This file is part of MlView
 *
 *MlView is free software; you can redistribute it and/or modify it under the terms of 
 *the GNU General Public License as published by the Free Software Foundation; either version 2, 
 *or (at your option) any later version.
 *
 *MlView is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *See the GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License along with MlView; 
 *see the file COPYING. If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 *Copyright 2001-2002 Dodji Seketeli.
 */

#ifndef __MLVIEW_XML_DOCUMENT_H__
#define __MLVIEW_XML_DOCUMENT_H__

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/

#include <libxml/tree.h>
#include <gnome.h>

#include "mlview-app-context.h"
#include "mlview-file-descriptor.h"
#include "mlview-parsing-utils.h"


#define MLVIEW_TYPE_XML_DOCUMENT (mlview_xml_document_get_type ())
#define MLVIEW_XML_DOCUMENT(object) (GTK_CHECK_CAST ((object), MLVIEW_TYPE_XML_DOCUMENT, MlViewXMLDocument))
#define MLVIEW_XML_DOCUMENT_CLASS(klass) (GTK_CHECK_CLASS_CAST ((klass), MLVIEW_TYPE_XML_DOCUMENT, MlViewXMLDocumentClass))
#define MLVIEW_IS_XML_DOCUMENT(object) (GTK_CHECK_TYPE ((object), MLVIEW_TYPE_XML_DOCUMENT))
#define MLVIEW_IS_XML_DOCUMENT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), MLVIEW_TYPE_XML_DOCUMENT))

	typedef struct _MlViewXMLDocument MlViewXMLDocument ;
	typedef struct _MlViewXMLDocumentClass MlViewXMLDocumentClass ;
	typedef struct _MlViewXMLDocumentPrivate MlViewXMLDocumentPrivate ;

	struct _MlViewXMLDocument {
		GtkObject object ;
		MlViewXMLDocumentPrivate *private;
	} ;

	struct _MlViewXMLDocumentClass {
		GtkObjectClass parent_class ;

		/*Tree Edition signals*/
		void (*document_changed) (MlViewXMLDocument * a_xml_doc,
					  gpointer a_user_data) ;

		void (*node_cut) (MlViewXMLDocument * a_xml_doc,
				  xmlNode *a_parent_node,
				  xmlNode * a_cut_node,
				  gpointer a_user_data) ;

		void (*prev_sibling_node_inserted) (MlViewXMLDocument *a_xml_doc,
						    xmlNode *a_sibling_node,
						    xmlNode *a_inserted_node,
						    gpointer a_user_data) ;
		
		void (*next_sibling_node_inserted) (MlViewXMLDocument *a_xml_doc,
						    xmlNode *a_sibling_node,
						    xmlNode *a_inserted_node,
						    gpointer a_user_data) ;

		void  (*child_node_added) (MlViewXMLDocument * a_xml_doc,
					   xmlNode *a_parent_node,
					   xmlNode * a_added_node,
					   gpointer a_user_data) ;

		void (*content_changed) (MlViewXMLDocument *a_xml_doc,
					 xmlNode *a_node,
					 gpointer a_user_data) ;

		void (*name_changed) (MlViewXMLDocument *a_xml_doc,
					 xmlNode *a_node,
					 gpointer a_user_data) ;

		void (*node_changed) (MlViewXMLDocument *a_xml_doc,
				      xmlNode *a_node,
				      gpointer a_user_data) ;
		
		void (*file_path_changed) (MlViewXMLDocument *a_xml_doc,
					   gpointer a_user_data) ;
		

	} ;

	guint 
	mlview_xml_document_get_type (void) ;

	xmlNode *
	mlview_xml_document_set_node_content (MlViewXMLDocument * a_xml_document,
					      xmlNode *a_node, gchar * a_content) ;

	xmlNode *
	mlview_xml_document_set_node_name (MlViewXMLDocument * a_xml_document,
					   xmlNode *a_node, gchar * a_name);
	MlViewXMLDocument * 
	mlview_xml_document_new (xmlDocPtr a_xml_doc, MlViewAppContext *a_context) ;	

	MlViewXMLDocument *  
	mlview_xml_document_open (gchar * a_file_name, MlViewAppContext *a_context) ;

	MlViewXMLDocument* 
	mlview_xml_document_clone (MlViewXMLDocument *a_original) ;

	MlViewFileDescriptor * 
	mlview_xml_document_get_file_descriptor (MlViewXMLDocument *a_xml_doc) ;

	xmlNode *
	mlview_xml_document_set_node_content (MlViewXMLDocument * a_xml_document,
					      xmlNode *a_node, gchar * a_content) ;

	void 
	mlview_xml_document_set_file_descriptor (MlViewXMLDocument *a_xml_doc, 
						 MlViewFileDescriptor *a_file_desc) ;

	void 
	mlview_xml_document_set_file_path (MlViewXMLDocument *a_xml_doc,
					   gchar *a_file_path) ;

	gchar * 
	mlview_xml_document_get_file_path (MlViewXMLDocument *a_xml_doc) ;

	MlViewAppContext *
	mlview_xml_document_get_app_context (MlViewXMLDocument * a_doc)  ;
	
	void
	mlview_xml_document_set_app_context (MlViewXMLDocument * a_doc, 
					     MlViewAppContext * a_context) ;

	xmlDocPtr 
	mlview_xml_document_get_xml_document (MlViewXMLDocument *a_xml_doc) ;

	void 
	mlview_xml_document_set_xml_document (MlViewXMLDocument *a_doc, 
					      xmlDocPtr a_mem_doc) ;

	void 
	mlview_xml_document_set_app_context (MlViewXMLDocument * a_doc, 
						  MlViewAppContext * a_context) ;

	gint 
	mlview_xml_document_associate_dtd_interactive (MlViewXMLDocument * a_doc) ;

	gint 
	mlview_xml_document_associate_dtd ( MlViewXMLDocument * a_doc,
					    MlViewExternalSubsetDefinition * a_ext_subs_def) ;

	gint 
	mlview_xml_document_validate (MlViewXMLDocument * a_doc) ;

	gboolean 
	mlview_xml_document_needs_saving (MlViewXMLDocument *a_doc) ;

	gint 
	mlview_xml_document_save (MlViewXMLDocument * a_doc) ;

	gint 
	mlview_xml_document_save_as (MlViewXMLDocument *a_doc, 
				     gchar * a_file_path) ;


/**************************************
 *Document Edition interfaces.
 ***************************************/
	void mlview_xml_document_copy_node_to_clipboard (xmlNode *a_xml_node, 
							 xmlDoc *a_doc) ;

	xmlNode * mlview_xml_document_add_child_node (MlViewXMLDocument *a_xml_doc,
						      xmlNode *a_parent_xml_node,
						      xmlNode *a_xml_node, 
						      gboolean a_subtree_required) ;

	xmlNode * mlview_xml_document_cut_node (MlViewXMLDocument * a_xml_doc, 
						xmlNode * a_xml_node) ;

	xmlNode * mlview_xml_document_insert_prev_sibling_node (MlViewXMLDocument *a_xml_document,
								xmlNode *a_sibling_node, xmlNode *a_xml_node,
								gboolean a_subtree_required) ;

	xmlNode * mlview_xml_document_insert_next_sibling_node (MlViewXMLDocument *a_xml_document,
								xmlNode *a_sibling_node, xmlNode *a_xml_node,
								gboolean a_subtree_required) ;

	void mlview_xml_document_paste_node_as_child (MlViewXMLDocument *a_xml_document,
						      xmlNode *a_parent_node) ;

	void mlview_xml_document_paste_node_as_sibling (MlViewXMLDocument * a_xml_document, xmlNode *a_parent_node,
							xmlNode * a_sibling_node, gboolean a_previous) ;

/*edition functions*/
#ifdef __cplusplus
}
#endif /*__cplusplus*/
#endif
